# -*- coding: utf-8 -*-
"""
Created on Mon Oct 18 22:48:47 2021

@author: Florian Janus
"""

import tkinter

class KnopfBeobachter:
    """
    * Die Schnittstellenbeschreibung für Klassen, die auf das Drücken des Knopfes reagieren wollen.
    """
    def Ausfuehren(self):
        """
        Wird aufgerufen, wenn der Knopf gedrückt wurde
        """
        pass


class Rahmen(KnopfBeobachter):
    """
    Ein einfaches Oberflächenbeispiel
    """

    def __init__(self):
        """
        Legt die Oberflächenelemente an
        """

        # Fenster
        self.fenster = tkinter.Tk()
        self.fenster.geometry("800x600")
        self.fenster.title = "Beispiel 1"

        # Anzeigeelement
        self.figur1 = Knopf(self, self.fenster, x=50, y=50, width=100, height=100)

        # Anzeigeelement
        self.figur2 = DoppelKnopf(self.fenster, x=250, y=50, width=100, height=100)

        self.fenster.mainloop()

    def Ausfuehren(self):
        """
        Aktionsmethode [mit Zusatzinformation]
        """
        self.figur1.place(x=self.figur1.winfo_x() + 10, y=self.figur1.winfo_y() + 10)

class Knopf(tkinter.Canvas):
    """
    Eine einfache, selbst gezeichnete Figur
    """
    def __init__(self, beobachter, master, x, y, width, height, **kw):
        super(Knopf, self).__init__(master=master, bg="yellow", **kw)
        self.place(x=x, y=y, width=width, height=height)
        self.gedrueckt = False
        self.beobachter = beobachter
        self.paint(width, height)

        def MausGedrueckt(event):
            self.gedrueckt = True
            self.paint()

        def MausLosgelassen(event):
            self.gedrueckt = False
            self.paint()
            if 0 <= event.x <= self.winfo_width() and 0 <= event.y <= self.winfo_height():
                self.beobachter.Ausfuehren()
        self.bind('<ButtonPress-1>', MausGedrueckt)
        self.bind('<ButtonRelease-1>', MausLosgelassen)

    def paint(self, width=None, height=None):
        """
        Zeichenmethode zum Darstellen des Knopfes
        """

        if width is None:
            width = self.winfo_width()
        if height is None:
            height = self.winfo_height()
        self.delete("all")
        self.create_rectangle(0, 0, width, height, fill="blue")
        self.create_rectangle(3, 3, width-6, height-6, fill="#e2e2e2" if self.gedrueckt else "#9d9c9c")


class DoppelKnopf(tkinter.Canvas):
    """
    Eine einfache, selbst gezeichnete Figur
    """

    def __init__(self, master, x, y, width, height, **kw):
        super(DoppelKnopf, self).__init__(master=master, bg="yellow", **kw)
        self.place(x=x, y=y, width=width, height=height)
        self.gedrueckt = False
        self.invertiert1 = False
        self.invertiert2 = False
        self.xKlick = -1
        self.paint(width, height)

        def MausGedrueckt(event):
            self.gedrueckt = True
            self.xKlick = event.x
            self.paint()

        def MausLosgelassen(event):
            self.gedrueckt = False
            if 0 <= event.x <= self.winfo_width() and 0 <= event.y <= self.winfo_height():
                if event.x <= self.winfo_width() // 2 and self.xKlick <= self.winfo_width() // 2:
                    self.invertiert1 = not self.invertiert1
                if event.x > self.winfo_width() // 2 and self.xKlick > self.winfo_width() // 2:
                    self.invertiert2 = not self.invertiert2
            self.paint()
        self.bind('<ButtonPress-1>', MausGedrueckt)
        self.bind('<ButtonRelease-1>', MausLosgelassen)

    def paint(self, width=None, height=None):
        """
        Zeichenmethode zum Darstellen des Doppelknopfes
        """
        if width is None:
            width = self.winfo_width()
        if height is None:
            height = self.winfo_height()
        self.delete("all")
        self.create_rectangle(0, 0, width // 2, height, fill="red" if self.invertiert1 else "green")
        self.create_rectangle(width // 2 + 1, 0, width, height, fill="blue" if self.invertiert2 else "yellow")
        if self.gedrueckt:
            if self.xKlick <= width // 2:
                self.create_rectangle(0, 0, width // 2, height, fill="#e2e2e2")
            else:
                self.create_rectangle(width // 2 + 1, 0, width, height, fill="#e2e2e2")
